unit convert_u;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, Grids, Menus,FileCtrl;

type
  tthread_spooler=class(tthread)
  protected
    procedure execute;override;
  public
    constructor create; virtual;
  end;

function file_in_use(fn:string):bool;
function shell_execute_wait(cmd,dir:string;var info:string):bool;
function spooler_work(flb:tfilelistbox;ffmpeg,param:string):integer;

var
  thread_spooler:tthread_spooler;

implementation

uses main_u;

//----------------------------------------------------------------
//thread funktionen
//----------------------------------------------------------------
constructor tthread_spooler.create;
begin
  //Thread soll sich nach Beendigung automatisch loeschen
  inherited create(false);
  freeOnTerminate:=true;
end;

procedure tthread_spooler.execute;
var
  rc:integer;
begin
  //Konvertirungs-Panel gruen faerben
  //datan wird erkannt, dass Konvertierung aktiv ist
  main_f.convert_working_p.color:=cllime;

  //konvertiere erstes File aus Konvertierungsspooler
  rc:=convert_u.spooler_work(
    main_f.convert_flb,
    main_f.homedir+'ffmpeg.exe',
    main_f.convert_param_cb.text
  );

  //passe Konvertierungs-Statistiken an
  case rc of
    -1:inc(main_f.convert_err_c);
     0:;
     1:inc(main_f.convert_mpg_c);
     2:inc(main_f.convert_m1v_c);
  end;

  //Konvertierungs-Panel wieder rot faerben
  main_f.convert_working_p.color:=clred;
end;

//----------------------------------------------------------------
//Konvertierungs-Funktionen
//----------------------------------------------------------------

//wird Datei gerade von einem anderen Prozess benutzt?
function file_in_use(fn:string):bool;
var
  HF:HFile;
  rc:byte;
begin
  result:=false;if not fileexists(fn) then exit;
  HF:=CreateFile(PChar(fn),GENERIC_READ or GENERIC_WRITE,0,nil,OPEN_EXISTING,0,0);
  rc:=(ord(HF=INVALID_HANDLE_VALUE));
  CloseHandle(HF);
  if rc=1 then result:=true;
end;

//warte gewisse Zeit, bis die Dateie von keinem
//Prozess mehr genutzt wird.
function file_blocked(fn:string):bool;
var
  r:integer;
begin
  r:=0;
  repeat
    application.processmessages;
    sleep(100);
    inc(r);
  until(not file_in_use(fn))or(r>1000);
  result:=(r>1000);
end;

//erzeuge einen neuen Windows-Prozess und warte, bis er beendet ist
function shell_execute_wait(cmd,dir:string;var info:string):bool;

  function lasterr:string;
  var
    l:dword;
  begin
    l:=getlasterror();
    result:='ERROR: '+inttostr(l);
  end;

const
  _strmax=500;
var
  str:pchar;
  tsi:TStartupInfo;
  tpi:TProcessInformation;
  dw,wrc:dword;
  pdir:pansichar;
begin
  //Prozess-Struktur vorbereiten
  FillChar(tsi,SizeOf(TStartupInfo),0);
  tsi.cb:=SizeOf(TStartupInfo);
  tsi.wShowWindow:=sw_hide;
  tsi.dwFlags:=STARTF_USESHOWWINDOW;

  //Gehe von Erfolg aus
  info:='OK '+cmd;

  //Arbeitsordner setzen
  if dir='' then pdir:=nil
            else pdir:=pansichar(dir);

  //Prozess erzeugen
  str:=allocmem(_strmax);
  try
    strpcopy(str,cmd);
    if CreateProcess(
      nil,
      str,
      nil,nil,False,
      0,
      nil,
      pdir,
      tsi,tpi
    )
    then begin

      //Warte maximal 5 Minuten auf Prozess-Ende
      wrc:=WaitForSingleObject(tpi.hProcess,60*5*1000);
      if wrc=WAIT_OBJECT_0 then begin
        //Prozess erfolgreich beendet

        //Hat auch alles geklappt?
        if GetExitCodeProcess(tpi.hProcess,dw) then begin
          if dw<>0 then begin
            info:='GetExitCodeProcess(): '+lasterr()+' '+cmd;
            result:=false;
            exit;
          end;
        end;
        CloseHandle(tpi.hProcess);
        CloseHandle(tpi.hThread);
      end
      else begin
        //es gab Probleme
        info:='WaitForSingleObject(): '+lasterr()+' '+cmd;
        result:=false;
        exit;
      end;
    end
    else begin
      info:='Konnte Prozess "'+cmd+'" nicht erzeugen - Abbruch! '+lasterr()+' '+cmd;
      result:=false;
      exit;
    end;
    result:=true;
  finally
    freemem(str);
  end;
end;

//Konvertiere FLV/WMV/MP4-Movie mittels FFMPEG.EXE entweder in
//eine MPG-Datei mit Sound oder eine M1V-Datei ohne Sound
//Ergebnis:
// 0: Misserfolg
// 1: MPG generiert
// 2: M1V generiert
function convert_movie_stream(ffmpeg,param,dir,fn_free:string):byte;
var
  fn_from,fn_to,cmd,errs:string;
begin

  //Quellnamen bilden, z.B. test.flv-
  fn_from:=dir+fn_free+'.flv'+_postfix_work;

  //Konvertierung: Probiere zuerst: test.flv- -> test.mpg-
  result:=1;
  fn_to:=dir+fn_free+'.mpg'+_postfix_work;
  cmd:=ffmpeg+' '+'-i "'+fn_from+'" '+param+' '+'"'+fn_to+'"';

  //hat es geklappt? Dann raus!
  if shell_execute_wait(cmd,dir,errs) then exit;

  //nein, hat nicht geklappt: Ergebnisfile lschen
  deletefile(fn_to);

  //und nochmal Konvertierung, diesmal test.flv- -> test.m1v-
  result:=2;
  fn_to:=changefileext(fn_to,'.m1v'+_postfix_work);
  cmd:=ffmpeg+' '+'-i "'+fn_from+'" '+'-an '+param+' '+'"'+fn_to+'"';

  //hat es geklappt? Dann raus!
  if shell_execute_wait(cmd,dir,errs) then exit;

  //nein, hat wieder nicht geklappt
  deletefile(fn_to);

  //Misserfolg mitteilen
  result:=0;
end;


//Lies FLV-Datei aus Spooler und konvertiere sie--------------------
//Ergebnis:
// -1: Fehler
//  0: nichts gemacht
//  1: MPG generiert
//  2: M1V generiert
function spooler_work(flb:tfilelistbox;ffmpeg,param:string):integer;
var
  dir,fn_free,ext,fn_from,fn_to:string;
  rc:integer;
  fhi,fho:integer;
begin
  result:=0;

  //Spooler leer? Dann raus!
  if flb.Items.count=0 then exit;

  //Erste Sppoler-Datei holen
  fn_from:=dir+flb.items[0];

  //Auf Freigabe warten
  if file_blocked(fn_from) then exit;

  //Konvertierung im Filenamen dokumentieren: test.flv+ -> test.flv-
  fn_to:=changefileext(fn_from,'.flv'+_postfix_work);
  if not renamefile(fn_from,fn_to) then exit;
  fn_from:=fn_to;

  //Konvertieren: test.flv- in test.mpg- oder test.m1v-
  dir:=flb.directory+'\';
  fn_free:=extractfilename(fn_from);
  fn_free:=changefileext(fn_free,'');
  rc:=convert_movie_stream(ffmpeg,param,dir,fn_free);
  if rc=0 then begin
    //Fehler: Konvertierung misslungen
    //test.flv- -> test#err.flv
    fn_to:=dir+fn_free+_flv_err;
    renamefile(fn_from,fn_to);
    result:=-1;
    exit;
  end;

  //Okay, Original-File retten: test.flv- -> test#org.flv
  fn_to:=dir+fn_free+_flv_org;
  renamefile(fn_from,fn_to);

  //Zielnamen anpassen: test.mpg- -> test.mpg oder test.m1v- -> test.m1v
  if rc=1 then ext:='.mpg' else ext:='.m1v';
  fn_from:=dir+fn_free+ext+_postfix_work;
  fn_to:=dir+fn_free+ext;
  renamefile(fn_from,fn_to);

  //Datum vom Ziel auf Quelle-Datum aendern
  fn_from:=dir+fn_free+_flv_org;
  fn_to:=dir+fn_free+ext;
  fhi:=FileOpen(fn_from,fmOpenREAD or fmShareDenyNone);
  fho:=FileOpen(fn_to,fmOpenWrite or fmShareDenyNone);
  FileSetDate(fho,filegetdate(fhi));
  closehandle(fho);
  closehandle(fhi);

  result:=rc;
end;

end.

