unit main_u;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, Grids, ComCtrls, FileCtrl, XPMan, inifiles;

const
  _cap='FLV-CacheCatchConverter V1.0';
  _inifn='flvccc.ini';

  _dir_work='work\';

  _postfix='+';
  _postfix_work='-';

  _flv_org='#org.flv';
  _flv_err='#err.flv';

  _color_on=clwindowtext;
  _color_off=cl3DDkShadow;

type
  Tmain_f = class(TForm)
    pctrl: TPageControl;
    cache_ts: TTabSheet;
    convert_ts: TTabSheet;
    cache_sg: TStringGrid;
    convert_flb: TFileListBox;
    XPManifest1: TXPManifest;
    timer_t: TTimer;
    cache_r_p: TPanel;
    cache_target_b: TButton;
    cache_target_e: TEdit;
    cache_l_p: TPanel;
    cache_refresh_b: TButton;
    cache_catch_b: TButton;
    cache_delete_b: TButton;
    cache_target_chb: TCheckBox;
    cache_sort_size_rb: TRadioButton;
    cache_sort_datetime_rb: TRadioButton;
    convert_p: TPanel;
    Label1: TLabel;
    convert_b: TButton;
    convert_param_cb: TComboBox;
    cache_ie_rb: TRadioButton;
    cache_ff_rb: TRadioButton;
    convert_auto_chb: TCheckBox;
    convert_working_p: TPanel;
    cache_lb: TListBox;
    cache_work_lb: TListBox;

    procedure FormCreate(Sender: TObject);
    procedure pctrlChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure timer_tTimer(Sender: TObject);

    procedure cache_refresh_bClick(Sender: TObject);
    procedure cache_catch_bClick(Sender: TObject);
    procedure cache_delete_bClick(Sender: TObject);
    procedure cache_target_chbClick(Sender: TObject);
    procedure cache_ie_rbClick(Sender: TObject);
    procedure cache_sort_datetime1Click(Sender: TObject);
    procedure cache_sgDblClick(Sender: TObject);
    procedure cache_target_bClick(Sender: TObject);

    procedure convert_bClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    homedir:string;

    cache_catch_c:integer;
    cache_check_c:integer;

    convert_err_c,
    convert_mpg_c,
    convert_m1v_c:integer;

    procedure error(s:string);
  end;

var
  main_f: Tmain_f;

implementation

{$R *.dfm}

uses cache_u,convert_u;

//----------------------------------------------------------------
//Allgemeine Funktionen
//----------------------------------------------------------------

procedure Tmain_f.FormCreate(Sender: TObject);
var
  inif:tinifile;
  obj:tobject;
begin
  formstyle:=fsstayontop;
  randomize;
  caption:=_cap;
  homedir:=extractfilepath(application.exename);

  //create work directory
  if not directoryexists(homedir+_dir_work) then mkdir(homedir+_dir_work);

  //read parameters
  inif:=tinifile.create(homedir+_inifn);
  try
    top:=inif.Readinteger('param','pos_top',100);
    left:=inif.Readinteger('param','pos_left',100);
    height:=inif.readinteger('param','pos_height',165);
    width:=inif.readinteger('param','pos_width',650);

    cache_ie_rb.checked:=inif.ReadBool('param','cache_ie',true);
    cache_ff_rb.checked:=inif.ReadBool('param','cache_ff',false);

    cache_sort_datetime_rb.checked:=inif.ReadBool('param','cache_sort_datetime',true);
    cache_sort_size_rb.checked:=inif.ReadBool('param','cache_sort_size',false);

    convert_auto_chb.checked:=inif.ReadBool('param','convert_auto',false);
    convert_param_cb.text:=inif.Readstring('param','convert_param',convert_param_cb.text);
    convert_param_cb.SelLength:=0;
  finally
    inif.free;
  end;

  //set tabsheets
  pctrl.align:=alclient;
  pctrl.ActivePage:=cache_ts;

  cache_catch_c:=0;
  cache_check_c:=0;
  cache_u.init(cache_sg);
  cache_target_chbClick(Sender);

  convert_err_c:=0;
  convert_mpg_c:=0;
  convert_m1v_c:=0;
  convert_flb.align:=alclient;

  convert_flb.mask:='*.flv'+_postfix;
  convert_flb.Directory:=homedir+_dir_work;
  convert_working_p.ParentBackground:=false;
  convert_working_p.color:=clred;

  //autostart for converter?
  obj:=nil;
  if convert_auto_chb.Checked then obj:=sender;
  convert_bClick(obj);

  timer_t.Tag:=-1;
  timer_t.Enabled:=true;
end;

procedure Tmain_f.FormDestroy(Sender: TObject);
var
  inif:tinifile;
begin
  //write parameters
  deletefile(homedir+_inifn);
  inif:=tinifile.create(homedir+_inifn);
  try
    inif.writeinteger('param','pos_top',top);
    inif.writeinteger('param','pos_left',left);
    inif.writeinteger('param','pos_height',height);
    inif.writeinteger('param','pos_width',width);

    inif.writeBool('param','cache_ie',cache_ie_rb.checked);
    inif.writeBool('param','cache_ff',cache_ff_rb.checked);

    inif.writeBool('param','cache_sort_datetime',cache_sort_datetime_rb.checked);
    inif.writeBool('param','cache_sort_size',cache_sort_size_rb.checked);

    inif.writeBool('param','convert_auto',convert_auto_chb.checked);
    inif.writestring('param','convert_param',convert_param_cb.text);
  finally
    inif.free;
  end;
end;

//Fehlermeldungen anzeigen------------------
procedure tmain_f.error(s:string);
begin
  beep;
  application.messagebox(pchar(s),'ERROR',mb_ok);
end;

//change active tabsheet----------------------------
procedure Tmain_f.pctrlChange(Sender: TObject);
begin
  if pctrl.activepage=cache_ts then begin
    cache_refresh_bClick(Sender);
    activecontrol:=cache_sg;
  end
  else begin
    activecontrol:=convert_flb;
  end;
end;

//timer-ereignis jede sekunden abarbeiten------------------------
procedure Tmain_f.timer_tTimer(Sender: TObject);
var
  s:string;
begin
  //erstaufruf?
  if timer_t.Tag=-1 then begin
    //jep, dann nimm refresh vom cache vor
    timer_t.Tag:=0;
    cache_refresh_bClick(Sender);
  end;

  //timer.tag wechsel jede sekunden von 0 auf 1 auf 0 usw.
  timer_t.tag:=timer_t.tag+1;if timer_t.tag>1 then timer_t.tag:=0;

  //Form.caption bilden
  s:='';
  if convert_working_p.color=cllime then begin
    //konvertierung gerade aktiv
    if timer_t.tag=0 then s:=s+'Working'
                     else s:=s+' ... ';
  end
  else begin
    //konvertierung inaktiv: statistik anzeigen
    s:=
      s+
      'MPG: '+inttostr(convert_mpg_c)+ ' '+
      'M1V: '+inttostr(convert_m1v_c)+' '+
      'ERR: '+inttostr(convert_err_c);
  end;

  //und form.caption setzen
  caption:=
    _cap+' | '+
    'CHECK: '+inttostr(cache_check_c)+
    ' CATCH: '+inttostr(cache_catch_c)+
    ' | '+
    s;

  //update des konvertierungs-spoolers
  convert_flb.update;

  //Konvertierungs-Spooler aktiv? Wenn nicht, dann raus!
  if convert_b.caption<>'STOP' then exit;

  //Wird gerade konvertiert? Dann raus!
  if convert_working_p.color=cllime then exit;

  //Okay, nchstes FLV im Spooler kann konvertiert werden
  thread_spooler:=tthread_spooler.Create;
end;

//----------------------------------------------------------------
//Cache-Funktionen
//----------------------------------------------------------------

//ie- /firefox-stringgrid mit flv-cache-files fuellen-------------
procedure Tmain_f.cache_refresh_bClick(Sender: TObject);
var
  cache_ok:bool;
begin
  cache_check_c:=cache_u.refresh(
    cache_sg,
    cache_lb,
    cache_work_lb,
    cache_ff_rb.checked,
    cache_sort_size_rb.checked
  );

  //setze buttons und farben gemss cache-status
  cache_ok:=(cache_sg.cells[ord(_cache_file),0]<>'');
  cache_catch_b.enabled:=cache_ok;
  cache_delete_b.enabled:=cache_ok;
  if cache_ok then cache_sg.color:=_color_on
              else cache_sg.color:=_color_off;
end;

//kopiere/verschiebe die selektierten files in der cache-stringgrid-------
procedure Tmain_f.cache_catch_bClick(Sender: TObject);
var
  target:string;
begin
  //speziellen zieldateinamen (target) verwenden?
  target:='';if cache_target_chb.Checked then target:=cache_target_e.text;
  cache_u.catch(
    cache_sg,
    homedir+_dir_work,
    target,
    cache_catch_c
  );
  cache_refresh_bClick(Sender);
end;

//lsche die selektierten files in der cache-stringgrid-------
procedure Tmain_f.cache_delete_bClick(Sender: TObject);
begin
  cache_u.delete(cache_sg);
  cache_refresh_bClick(Sender);
end;

//speziellen zieldateinamen verwenden?
procedure Tmain_f.cache_target_chbClick(Sender: TObject);
begin
  if cache_target_chb.checked then begin
    //ja, speziellen zieldateinamen (target) verwenden
    cache_target_e.Color:=_color_on;
    cache_target_e.enabled:=true;
    cache_target_b.enabled:=true;
  end
  else begin
    //nein, verwende originalen cache-dateinamen
    cache_target_e.Color:=_color_off;
    cache_target_e.enabled:=false;
    cache_target_b.enabled:=false;
  end;
end;

//Cache-Wechsel von IE auf Firefox bzw. umgekehrt-------------
procedure Tmain_f.cache_ie_rbClick(Sender: TObject);
begin
  cache_refresh_bClick(Sender);
end;

//Cache-Stringgrid umsortieren-------------------------------------
procedure Tmain_f.cache_sort_datetime1Click(Sender: TObject);
begin
  cache_refresh_bClick(Sender);
end;

//Doppelklick auf Cache-Eintrag: Kopieren/Verschieben in Spooler----------
procedure Tmain_f.cache_sgDblClick(Sender: TObject);
begin
  cache_catch_bClick(sender);
end;

//Einen zuflligen, speziellen Zieldateinamen bilden-----------
procedure Tmain_f.cache_target_bClick(Sender: TObject);
begin
  cache_target_e.text:=cache_u.target_name_random;
end;

//----------------------------------------------------------------
//Konvertierungs-Funktionen
//----------------------------------------------------------------

//Konvertierungsspooler aktivieren bzw. deaktivieren---------------
procedure Tmain_f.convert_bClick(Sender: TObject);
begin
  if (sender<>nil)and(convert_b.caption='Start') then begin
    //spooler aktivieren

    //existiert ffmpeg.exe?
    if not fileexists(homedir+'ffmpeg.exe')then begin
      //nein
      error('FFMPEG.EXE "'+homedir+'ffmpeg.exe" not found!');
      exit;
    end;

    //okay, spooler wird aktiviert
    convert_b.caption:='STOP';
    convert_flb.Color:=_color_on;
    convert_param_cb.Enabled:=false;
    convert_param_cb.color:=_color_off;
  end
  else begin
    //spooler deaktivieren
    convert_b.caption:='Start';
    convert_flb.Color:=_color_off;
    convert_param_cb.Enabled:=true;
    convert_param_cb.color:=_color_on;
  end;
end;

end.

